﻿using System.Threading;
using System.Threading.Tasks;
using InternshipSystem.Api.Queries;
using InternshipSystem.Api.Security;
using InternshipSystem.Core.Commands;
using InternshipSystem.Repository;
using Microsoft.AspNetCore.Authorization;
using InternshipSystem.Api.Security;
using InternshipSystem.Api.Services;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;

namespace InternshipSystem.Api.Controllers
{
    [Route("internshipRegistration")]
    public class InternshipRegistrationController : ControllerBase
    {
        private readonly IInternshipService _internshipService;

        public InternshipRegistrationController(IInternshipService internshipService)
        {
            _internshipService = internshipService;
        }

        /// <summary>
        /// Validate and add filled internship registration form 
        /// </summary>
        /// <param name="updateRegistration">Internship registration data</param>
        /// <response code="200">If registration form was successfully added</response>
        /// <response code="400">If the provided registration query was malformed</response>
        /// <response code="401">This action is only available for authorized student registered for current edition</response>
        [HttpPut]
        [ProducesResponseType(StatusCodes.Status200OK)]
        [ProducesResponseType(StatusCodes.Status400BadRequest)]
        [ProducesResponseType(StatusCodes.Status401Unauthorized)]
        [Authorize]
        public async Task<ActionResult> SubmitRegistrationForm([FromBody] RegistrationFormQuery registrationQuery, 
            [FromServices] User user, CancellationToken cancellationToken)
        {
            var validator = new RegistrationFormQuery.Validator();
            var validationResult = await validator.ValidateAsync(registrationQuery, cancellationToken);

            if (!validationResult.IsValid)
            {
                return BadRequest(validationResult.ToString());
            }

            return await _internshipService.SubmitRegistration(registrationQuery, user.PersonNumber, cancellationToken);
        }
    }
}