using Microsoft.EntityFrameworkCore;
using InternshipSystem.Core;
using InternshipSystem.Core.Entity.Internship;
using InternshipSystem.Core.UglyOrmArtifacts;

namespace InternshipSystem.Repository
{
    public class InternshipDbContext : DbContext
    {
        public DbSet<Company> Companies { get; set; }
        public DbSet<Edition> Editions { get; set; }
        public DbSet<StaticPage> StaticPages { get; set; }
        public DbSet<InternshipType> InternshipTypes { get; set; }
        public DbSet<Student> Students { get; set; }

        public InternshipDbContext(DbContextOptions<InternshipDbContext> options)
            : base(options)
        {          
        }

        protected override void OnConfiguring(DbContextOptionsBuilder optionsBuilder) =>
            optionsBuilder
                .UseSnakeCaseNamingConvention();

        protected override void OnModelCreating(ModelBuilder modelBuilder)
        {
            modelBuilder.Entity<BranchOffice>()
                .OwnsOne(bo => bo.Address);

            modelBuilder.Entity<InternshipProgram>()
                .OwnsOne(ip => ip.Mentor);

            modelBuilder.Entity<ProgramSubject>(builder =>
            {
                builder
                    .HasKey(subject => new { subject.InternshipProgramId, subject.InternshipSubjectId });
                
                builder
                    .HasOne(k => k.Program)
                    .WithMany(model => model.ChosenSubjects)
                    .HasForeignKey(subject => subject.InternshipProgramId);
                
                builder
                    .HasOne(k => k.Subject)
                    .WithMany()
                    .HasForeignKey(subject => subject.InternshipSubjectId);
            });
            
            modelBuilder.Entity<EditionSubject>(builder =>
            {
                builder
                    .HasKey(subject => new { subject.EditionId, subject.InternshipSubjectId});
                
                builder
                    .HasOne(k => k.Edition)
                    .WithMany(model => model.AvailableSubjects)
                    .HasForeignKey(p => p.EditionId);

                builder
                    .HasOne(k => k.Subject)
                    .WithMany()
                    .HasForeignKey(subject => subject.InternshipSubjectId);
            });
        }
    }
}